
package com.reactlibrary;

import android.app.Activity;
import android.database.ContentObserver;
import android.database.Cursor;
import android.net.Uri;
import android.os.Handler;
import android.os.HandlerThread;
import android.provider.MediaStore;
import android.util.Log;

import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;

import java.util.Timer;
import java.util.TimerTask;

public class RNCbmScreenShotModule extends ReactContextBaseJavaModule {
  private static final String TAG = "RNCbmScreenShot";

  private final ReactApplicationContext reactContext;
  private static Activity ma;

  private static final String NAVIGATION = "navigationBarBackground";

  private static final String[] KEYWORDS = {
      "screenshot", "screen_shot", "screen-shot", "screen shot",
      "screencapture", "screen_capture", "screen-capture", "screen capture",
      "screencap", "screen_cap", "screen-cap", "screen cap", "Screenshot", "截屏", "截图"
  };

  /**
   * 读取媒体数据库时需要读取的列
   */
  private static final String[] MEDIA_PROJECTIONS = {
      MediaStore.Images.ImageColumns.DATA,
      MediaStore.Images.ImageColumns.DATE_TAKEN,
  };

  /**
   * 外部存储器内容观察者
   */
  private ContentObserver mExternalObserver;
  private HandlerThread mHandlerThread;
  private Handler mHandler;


  /**
   * 内部存储器内容观察者
   */
  private ContentObserver mInternalObserver;

  public RNCbmScreenShotModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
  }

  @Override
  public String getName() {
    return "RNCbmScreenShot";
  }

  public static void initScreenShotShareSDK(Activity activity){
    ma = activity;
  }

  @ReactMethod
  public void startListener() {
    mHandlerThread = new HandlerThread("Screenshot_Observer");
    mHandlerThread.start();
    mHandler = new Handler(mHandlerThread.getLooper());

    // 初始化
    mInternalObserver = new MediaContentObserver(MediaStore.Images.Media.INTERNAL_CONTENT_URI, mHandler);
    mExternalObserver = new MediaContentObserver(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, mHandler);

    // 添加监听
    this.reactContext.getContentResolver().registerContentObserver(
        MediaStore.Images.Media.INTERNAL_CONTENT_URI,
        true,
        mInternalObserver
    );

    this.reactContext.getContentResolver().registerContentObserver(
        MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
        true,
        mExternalObserver
    );
  }

  @ReactMethod
  public void stopListener() {
    this.reactContext.getContentResolver().unregisterContentObserver(mInternalObserver);
    this.reactContext.getContentResolver().unregisterContentObserver(mExternalObserver);
  }

  @ReactMethod
  public void hasNavigationBar(Promise promise) {
    // boolean navigationBarExisted = isNavigationBarExist(ma);
    // promise.resolve(navigationBarExisted);
  }

  /**
   * 媒体监听
   */
  private class MediaContentObserver extends ContentObserver {
    private Uri mContentUri;

    public MediaContentObserver(Uri contentUri, Handler handler) {
      super(handler);
      mContentUri = contentUri;
    }

    @Override
    public void onChange(boolean selfChange) {
      super.onChange(selfChange);
      Log.d(TAG, mContentUri.toString());
      Timer timer = new Timer();
      TimerTask task = new TimerTask() {
        @Override
        public void run() {
          handleMediaContentChange(mContentUri);
        }
      };
      timer.schedule(task, 1000);
    }
  }

  public void handleMediaContentChange(Uri contentUri) {
    Cursor cursor = null;
    try {
      // 数据改变时查询数据库中最后加入的一条数据
      cursor = this.reactContext.getContentResolver().query(
          contentUri,
          MEDIA_PROJECTIONS,
          null,
          null,
          MediaStore.Images.ImageColumns.DATE_ADDED + " desc limit 1"
      );

      if (cursor == null) {
        return;
      }
      if (!cursor.moveToFirst()) {
        return;
      }
      // 获取各列的索引
      int dataIndex = cursor.getColumnIndex(MediaStore.Images.ImageColumns.DATA);
      int dateTakenIndex = cursor.getColumnIndex(MediaStore.Images.ImageColumns.DATE_TAKEN);

      // 获取行数据
      String data = cursor.getString(dataIndex);
      long dateTaken = cursor.getLong(dateTakenIndex);
      // 处理获取到的第一行数据
      handleMediaRowData(data, dateTaken);
    } catch (Exception e) {
      WritableMap map = Arguments.createMap();
      map.putInt("code", 500);
      sendEvent(this.reactContext, "ScreenShotShare", map);
      e.printStackTrace();
    } finally {
      if (cursor != null && !cursor.isClosed()) {
        cursor.close();
      }
    }
  }

  public void sendEvent(ReactContext reactContext, String eventName, @Nullable WritableMap params) {
    reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class).emit(eventName, params);
  }

  // 当前APP内容截图
//  private static Bitmap normalShot(Activity activity) {
//    View decorView = activity.getWindow().getDecorView();
//    decorView.setDrawingCacheEnabled(true); // 设置缓存，可用于实时截图
//    decorView.buildDrawingCache();
//
//    Rect outRect = new Rect();
//    decorView.getWindowVisibleDisplayFrame(outRect);
//    int statusBarHeight = outRect.top;//状态栏高度
//
//    Bitmap bitmap = Bitmap.createBitmap(decorView.getDrawingCache(),
//        0, statusBarHeight,
//        decorView.getMeasuredWidth(), decorView.getMeasuredHeight() - statusBarHeight);
//
//    decorView.setDrawingCacheEnabled(false);
//    decorView.destroyDrawingCache();
//    return bitmap;
//  }

  // 判断全面屏虚拟导航栏是否存在
//  public static boolean isNavigationBarExist(Activity activity) {
//    ViewGroup vp = (ViewGroup) activity.getWindow().getDecorView();
//    if (vp != null) {
//      for (int i = 0; i < vp.getChildCount(); i++) {
//        vp.getChildAt(i).getContext().getPackageName();
//        if (vp.getChildAt(i).getId() != NO_ID && NAVIGATION.equals(activity.getResources().getResourceEntryName(vp.getChildAt(i).getId()))) {
//          return true;
//        }
//      }
//    }
//    return false;
//  }

  /**
   * 处理监听到的资源
   */
  private void handleMediaRowData(String data, long dateTaken) {
    Log.d(TAG, "处理资源"+data);
    if (checkScreenShot(data, dateTaken)) {
      WritableMap map = Arguments.createMap();
      map.putInt("code", 200);
      map.putString("uri", data);
      map.putString("from", "相册");
      sendEvent(this.reactContext, "ScreenShotShare", map);
      Log.d("=====", "新代码");
    } else {
      Log.d(TAG, "Not screenshot event");
      WritableMap map = Arguments.createMap();
      map.putInt("code", 500);
      sendEvent(this.reactContext, "ScreenShotShare", map);
    }
  }

  /**
   * 判断是否是截屏
   */
  private boolean checkScreenShot(String data, long dateTaken) {
    data = data.toLowerCase();
    Log.d(TAG, data);
    // 判断图片路径是否含有指定的关键字之一, 如果有, 则认为当前截屏了
    for (String keyWork : KEYWORDS) {
      if (data.contains(keyWork)) {
        return true;
      }
    }
    return false;
  }

  // 获取当前APP图片存储路径
//  private String getSystemFilePath() {
//    String cachePath;
//    if (Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState())
//        || !Environment.isExternalStorageRemovable()) {
//      cachePath = reactContext.getExternalFilesDir(Environment.DIRECTORY_PICTURES).getAbsolutePath();
//      //cachePath = context.getExternalCacheDir().getPath(); // 返回文件 uri，而非path
//    } else {
//      cachePath = reactContext.getFilesDir().getAbsolutePath();
//      //cachePath = context.getCacheDir().getPath(); // 返回文件 uri，而非path
//    }
//    return cachePath;
//  }

  // 保存截屏的bitmap为图片文件并返回路径
//  private void saveBitmap(Bitmap bitmap){
//    Long time = System.currentTimeMillis();
//    String path = getSystemFilePath() + "/screen-shot-catch" + time + ".png";
//    Log.d(TAG, path);
//    File filePic;
//    WritableMap map = Arguments.createMap();
//    try{
//      filePic = new File(path);
//      if (!filePic.exists()) {
//        filePic.getParentFile().mkdirs();
//        filePic.createNewFile();
//      }
//      FileOutputStream fos = new FileOutputStream(filePic);
//      bitmap.compress(Bitmap.CompressFormat.PNG, 100, fos);
//      fos.flush();
//      fos.close();
//      map.putInt("code", 200);
//      map.putString("uri", filePic.getAbsolutePath());
//      sendEvent(this.reactContext, "Screenshotcatch", map);
//      // 强制关闭软键盘
//      try{
//        ((InputMethodManager) ma.getSystemService(reactContext.INPUT_METHOD_SERVICE)).hideSoftInputFromWindow(ma.getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
//      }catch(java.lang.NullPointerException e){
//        Log.d(TAG, e.toString());
//      }
//    }catch(IOException e){
//      Log.d(TAG, e.toString());
//      e.printStackTrace();
//      map.putInt("code", 500);
//      sendEvent(this.reactContext, "Screenshotcatch", map);
//    }
//  }


}